#include<string.h>
#include<stdlib.h>

#include "measure.hh"


// fill int array with random numbers from [0, 100]
void fillData(int* aArray, int aSize) {
  for (int i = 0; i < aSize; ++i) {
    aArray[i] = rand() % 101;   
  }
}

// select array elements less than aValue w/ branching
size_t selectBranch(int* aInput, int* aOutput, size_t aSize, int aValue) {
   size_t j = 0;
   for (size_t i = 0; i < aSize; ++i) {
     if (aInput[i] <= aValue) {
       aOutput[j++] = aInput[i];
     }
   }
   return j;
}

// select array elements less than aValue w/o branching (predicated)
size_t selectPredicated(int* aInput, int* aOutput, size_t aSize, int aValue) {
   size_t j = 0;
   for (size_t i = 0; i < aSize; ++i) {
     aOutput[j] = aInput[i];
     j += (aInput[i] <= aValue);
   }
   return j;
}

int main(int argc, char* argv[]) {
  int lSize = 1000000;
  // comparison value for predicate 
  int lValue = 50;
  // run code w/ or w/o branch?
  bool lPredicated = false;

  if (argc > 1) {
    for (int i = 1; i < argc; ++i) {

      if (!strcmp(argv[i], "-p")) {
        lPredicated = true;
      }

      if (!strcmp(argv[i], "-n")) {
        lSize = atol(argv[++i]);
      }

      if (!strcmp(argv[i], "-v")) {
        lValue = atol(argv[++i]);
      }
    }   
  }

  // allocate input and output arrays
  int* lInput = new int[lSize];
  int* lOutput = new int[lSize];
  //fill input array
  fillData(lInput, lSize);

  double lTime = 0;
  size_t lRepeat = 100;
  size_t lNumQualifying = 0;
  for (size_t i = 0; i < lRepeat; ++i) {
  Measure lMeasure;
    if (lPredicated) {
      lMeasure.start();
      lNumQualifying = selectPredicated(lInput, lOutput, lSize, lValue);
      lMeasure.stop();
    }
    else {
      lMeasure.start();
      lNumQualifying = selectBranch(lInput, lOutput, lSize, lValue);
      lMeasure.stop();
    }
    lTime += lMeasure.mCpuTime();
  }
  lTime /= lRepeat;
  // print runtime
  std::cout 
    << "Selectivity " 
    << std::setw(4)  << std::setprecision(2) << std::fixed << (float)lValue/100 << ": " 
    << std::setw(8)  << std::setprecision(6) << std::fixed << lTime << " s" 
    << std::setw(16) << "[" << lNumQualifying << "/" << lSize << "]"
    << std::endl;

  // release memory
  delete[] lInput;
  delete[] lOutput;

  return 0;
}
